' =============================================================================
' TITLE:	MSR-H01 Hexapod & HexEngine Binary object avoidance
' AUTHOR:	James Dickinson
' DATE:		28/06/2011
' REVISION:	1.0
' 
' =============================================================================

' CONSTANTS ===================================================================
'
' ANY CONSTANTS USED FOR HEAD, BODY OR WALK CAN HAVE A MAX OR MIN VALUE OF -127/127

' TUNE THESE FOR FASTER RESPONSE OR TO REDUCE OSCILATION. 
' INCREASE VALUE TO DECREASE SENSITIVITY.
cPAN_SENS = 1800
cTILT_SENS = 1500

' MAXIMUM WALK AND TURN SPEED VERY MUCH DEPEND ON THE GAIT USED.
cMAX_WALK_SPEED	= 70	' MAXIMUM OF 127
cMAX_TURN_SPEED	= 60	' MAXIMUM OF 127

' HEAD LIMITS MAXIMUM OF +/- 127
' IF YOU NEED TO INVERT THE HEAD MOVMENT, CHANGE TEH SIGNS ON MAX & MIN
cMAX_HEAD_PAN = 127
cMIN_HEAD_PAN = -127
cMAX_HEAD_TILT = 100
cMIN_HEAD_TILT = -100

' BODY LIMITS MAXIMUM OF +/- 127
cMAX_BODY_Y_TRANSLATE = 100
cMIN_BODY_Y_TRANSLATE = -100
cMAX_BODY_X_ROTATE = 100
cMIN_BODY_X_ROTATE = -70
cMAX_BODY_Z_ROTATE = 100
cMIN_BODY_Z_ROTATE = -100

PIPPan = "~"
PIPBody = "~"


' VARIABLES ===================================================================

' get the current center of the image
midx = GetVariable("IMAGE_WIDTH") / 2
midy = GetVariable("IMAGE_HEIGHT") / 2

' PIP 'H' COMMAND VARIABLES
VariableH1 = 1
VariableH2 = 1

' PIP 'B' COMMAND VARIABLES
VariableB1 = 1
VariableB2 = 1
VariableB3 = 1
VariableB4 = 1
VariableB5 = 1
VariableB6 = 1

' PIP 'M' COMMAND VARIABLES
VariableM1 = 1
VariableM2 = 1
VariableM3 = 1
	
' WALK AND TURN SPEED
gWalk = 0
gTurn = 0
gTilt = 0
gPan = 0

' BODY ROTATE & TRANSLATE
gBodyXRotate = 0
gBodyZRotate = 0
gBodyYTranslate = 0

gPowerState = GetVariable("POWER_STATE")

'=======BINARY OBJECT AVOIDANCE START

width = GetVariable("IMAGE_WIDTH")
height = GetVariable("IMAGE_HEIGHT")

'Grab information from Point Location Plugin
furthest_x = GetVariable("FURTHEST_X")
furthest_y = GetVariable("FURTHEST_Y")
left_highest_x = GetVariable("HIGHEST_LEFTMOST_X")
left_highest_y = GetVariable("HIGHEST_LEFTMOST_Y")
right_highest_x = GetVariable("HIGHEST_RIGHTMOST_X")
right_highest_y = GetVariable("HIGHEST_RIGHTMOST_Y")

'set constants
dangerx = GetVariable("NEAREST_X")
dangery = GetVariable("NEAREST_Y")

too_close_dist = height - (height * 0.15)

if (dangery < too_close_dist) then
' nothing is danger close
  if (furthest_y < (width / 2)) then
  ' Furthest is valid
    gWalk = Interpolate(((width/2) - furthest_y), 0, (width/2), 0, cMAX_WALK_SPEED)
    gTurn = Interpolate(furthest_x,0,width,-cMAX_TURN_SPEED, cMAX_TURN_SPEED)
  else
    ' Furthest is too close (this needs to be expanded to look left/right with pan first)
    gWalk = 0;
    if (left_highest_y <= right_highest_y) then
      gTurn = Interpolate(left_highest_x,0,width,-cMAX_TURN_SPEED, cMAX_TURN_SPEED)
    else
      gTurn = Interpolate(left_highest_x,0,width,-cMAX_TURN_SPEED, cMAX_TURN_SPEED)
    end if
  end if
else
  'back away from danger point
  gWalk = -20;
  gTurn = Interpolate((width - danger_x),0,width,-cMAX_TURN_SPEED, cMAX_TURN_SPEED)
end if

'=======BINARY OBJECT AVOIDANCE END


' PIP HEAD PAN&TILT =================================================================
' Set Internal & External Variables for 'H' command (Head Pan/Tilt)
VariableH1 = (256 + int(-gPan)) and 255 
VariableH2 = (256 + int(-gTilt)) and 255 
if VariableH1 = 0 then VariableH1 = 1
if VariableH2 = 0 then VariableH2 = 1
ChkSumH = 255 - ((asc("H") + VariableH1 + VariableH2) and 255)
PIPPan = "~" & chr(3) & "H" & chr(VariableH1) & chr(VariableH2) & chr(ChkSumH)
setvariable "PIPPan", PIPPan


' Calculate the desired Body X & Z Rotate (X = Tilt, Z = Pan)
gBodyXRotate = Interpolate( gTilt, cMIN_HEAD_TILT, cMAX_HEAD_TILT, cMIN_BODY_X_ROTATE, cMAX_BODY_X_ROTATE )
gBodyZRotate = Interpolate(	gPan, cMIN_HEAD_PAN, cMAX_HEAD_PAN, cMIN_BODY_Z_ROTATE, cMAX_BODY_Z_ROTATE )


' PIP BODY ROTATE/TRANSLATE ========================================================
' Set Internal & External Variables for 'B' command (Body Pan/Tilt & Translate)
VariableB1 = (256 + int(gBodyXRotate)) and 255
VariableB3 = (256 + int(gBodyZRotate)) and 255
if gWalkEnable = 0 then
	VariableB5 = (256 + int(gBodyYTranslate)) and 255
end if
if VariableB1 = 0 then VariableB1 = 1
if VariableB3 = 0 then VariableB3 = 1
if VariableB5 = 0 then VariableB5 = 1
ChkSumB = 255 - ((asc("B") + VariableB1 + VariableB2 + VariableB3 + VariableB4 + VariableB5 + VariableB6) and 255)
PIPBody = "~" & chr(7) & "B" & chr(VariableB1) & chr(VariableB2) & chr(VariableB3) & chr(VariableB4) & chr(VariableB5) & chr(VariableB6) & chr(ChkSumB)
setvariable "PIPBody", PIPBody


' PIP WALK ========================================================================
' Get the WALK_ENABLE variable
gWalkEnable = GetVariable("WALK_ENABLE")

if ( gWalkEnable <> 0 ) then
'	VariableM1 = 1 ' Crab not used
	VariableM2 = (256 + int(gWalk)) and 255
	VariableM3 = (256 + int(gTurn)) and 255
end if

if VariableM2 = 0 then VariableM2 = 1
if VariableM3 = 0 then VariableM3 = 1


setvariable "GWALK", gWalk
setvariable "GTURN", gTurn
ChkSumM = 255 - ((asc("M") + VariableM1 + VariableM2 + VariableM3) and 255)
if gWalkEnable = 0 or (VariableM1 = 1 and VariableM2 = 1 and VariableM3 = 1) then
	PIPMove = ""
else
	PIPMove = "~" & chr(4) & "M" & chr(VariableM1) & chr(VariableM2) & chr(VariableM3) & chr(ChkSumM)
end if
setvariable "PIPMove", PIPMove




' PIP WAKE COMMAND ========================================================================

gPowerOn = GetVariable("POWER_ON")
gPowerOff = GetVariable("POWER_OFF")
PIPCommand = ""
if gPowerOn then
	PIPCommand = "~" & chr(1) & "+" & chr(212)
	gPowerOn = 0
	gPowerState = 1
	setvariable "POWER_ON", gPowerOn
end if

if gPowerOff then
	PIPCommand = "~" & chr(1) & "-" & chr(210)
	gPowerOff = 0
	gPowerState = 0
	gWalkEnable = 0
	SetVariable "WALK_ENABLE", gWalkEnable
	setvariable "POWER_OFF", gPowerOff
end if

setvariable "PIPCommand", PIPCommand
setvariable "POWER_STATE", gPowerState






' =============================================================================
' FUNCTIONS
' =============================================================================

' LINEAR INTERPOLATION FUNCTION
function Interpolate( pIn, pInLow, pInHigh, pOutLow, pOutHigh )

if pIn > pInHigh then 
	Interpolate = pOutHigh 
elseif pIn < pInLow then 
	Interpolate = pOutLow 
else 	
	Interpolate = ( pOutLow + ( ( (pIn - pInLow) * (pOutHigh - pOutLow) ) / (pInHigh - pInLow) ) )
end if
	
end function


